
#include "uCam.h"


// Set up some memory to hold the commands
const unsigned char SYNC[] = {0xAA, 0x0D, 0x00, 0x00, 0x00, 0x00};
const unsigned char ACK[] = {0xAA, 0x0E, 0x0D, 0x00, 0x00, 0x00};
const unsigned char BAUD_2400 [] = {0xAA, 0x07, 0x1F, 0x2F, 0x00, 0x00};
const unsigned char BAUD_4800 [] = {0xAA, 0x07, 0x1F, 0x17, 0x00, 0x00};
const unsigned char BAUD_9600 [] = {0xAA, 0x07, 0x1F, 0x0B, 0x00, 0x00};
const unsigned char BAUD_19200[] = {0xAA, 0x07, 0x1F, 0x05, 0x00, 0x00};
const unsigned char BAUD_38400[] = {0xAA, 0x07, 0x1F, 0x02, 0x00, 0x00};
const unsigned char BAUD_57600[] = {0xAA, 0x07, 0x1F, 0x01, 0x00, 0x00};
const unsigned char BAUD_115200[] = {0xAA, 0x07, 0x1F, 0x00, 0x00, 0x00};
const unsigned char BAUD_153600[] = {0xAA, 0x07, 0x07, 0x02, 0x00, 0x00};
const unsigned char BAUD_230400[] = {0xAA, 0x07, 0x07, 0x01, 0x00, 0x00};
const unsigned char BAUD_460800[] = {0xAA, 0x07, 0x07, 0x00, 0x00, 0x00};
const unsigned char BAUD_921600[] = {0xAA, 0x07, 0x01, 0x01, 0x00, 0x00};
const unsigned char BAUD_1228800[] = {0xAA, 0x07, 0x02, 0x00, 0x00, 0x00};
const unsigned char BAUD_1843200[] = {0xAA, 0x07, 0x01, 0x00, 0x00, 0x00};
const unsigned char BAUD_3686400[] = {0xAA, 0x07, 0x00, 0x00, 0x00, 0x00};
const unsigned char GET_PICTURE[] = {0xAA, 0x04, 0x02, 0x00, 0x00, 0x00};   // take snapshot picture raw mode

int picture_size;       // To track the current picture size


uCam::uCam(HardwareSerial *camPort,unsigned long baud) // constructor

{	
	_baud=9600;
	switch (baud)
	{
	case 2400 :{_baud=2400;break; }
	case 4800 :{_baud=4800;break;}
	case 9600 :{_baud=9600;break; }
	case 19200 : {_baud=19200;break;}
	case 38400 : {_baud=38400;break;}
	case 57600 : {_baud=57600;break;}
	case 115200 :{ _baud=115200;break;}
			
	}
	_uCam_Serial=camPort; // sets the serial port choses for the camera (hardware ones)
	
}

// Waits for an response from the uCam module and checks that a valid ACK was received
boolean uCam::Get_Response(byte type, byte command) {
	// First wait for the uCam module to send the reply or timeout
	// This waits for the module to become readable or timesout after 50 ms
	int times = 0;         // Variable to check for timeouts
	do 
	{
		delay(1);        // Wait 1 ms
		times++;            // Inc variable
	} 
	while ((times < 100 ) && (_uCam_Serial->available()==0));
	// Check if it timed out
	
	if (times > 99) 
	{
		return(false); // error et exit function
	}
	
	// Get the reply from the uCam module (will be 6 bytes
	unsigned char reply[6] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
	for (int i=0; i<6; i++) 
	{
		if (_uCam_Serial->available()>0) 
		{reply[i]= _uCam_Serial->read();}
		else
		{i--;}
	}
	
	// Check reply was valid, A valid ACK should be 0xAA, 0x0E, Command ID, X, 0x00, 0x00
	if (reply[0] == 0xAA && reply[1] == type && reply[2] == command && reply[4] == 0x00 && reply[5] == 0x00)
	// It was an ACK so return true
	{
		return(true); // Ok result and exit
	}
	else 
	{
		// Reply was invalid so print the reply and return false
		return(false); //non Ok and exit
	}
}

void uCam::begin() // start the camera communication with correct parameters between the appliances

{
  _uCam_Serial->begin(_baud); // sets the correct speed as declear on creation
  delay(100);
  SetBaud(_baud); // sends the camera the correc instruction
 }
 
boolean uCam::Sync() { 
	boolean result=false;
	// This will give 60 attempts to sync with the uCam module
	
	for (int nb=0; nb<=60; nb ++) 
	{
		
		//Serial.println(nb);
	
		// Send out the sync command
		_uCam_Serial->begin(_baud); delay(100);
		
		for (int i=0; i<6; i++) 
		{
			_uCam_Serial->write(SYNC[i]);
		}
		// Check if the response was an ACK
		if (Get_Response(_ACK,SYNC[1])) 
		{
			// It was an ACK so now get the next response - it should be a sync
			if (Get_Response(_SYNC,0x00)) 
			{
				
				// We need a small delay (1ms) from receiving the SYNC response and sending an ACK in return
				delay(1);
				for (int j=0; j<6; j++) 
				{
					_uCam_Serial->write(ACK[j]);
				}
				// Everything is now complete so return true
				result=true;
				nb=61;
			}
		}
		// Wait a while and try again
		if (!result) {delay(10); };
		
	}
	// Something went wrong so return false

	//Serial.println(result);

	if (result) {return(true);} else {return (false);}
}

boolean uCam::SetBaud(int baud) {
	// Send out the set baud command
	for (int i=0; i<6; i++) 
	{
		// Select the correct data for the desired baud
		switch (baud)
		{
		case 2400 :{ _uCam_Serial->write(BAUD_2400[i]);break;}
		case 4800 :{ _uCam_Serial->write(BAUD_4800[i]);break;}
		case 9600 :{ _uCam_Serial->write(BAUD_9600[i]);break;}
		case 19200 :{ _uCam_Serial->write(BAUD_19200[i]);break;}
		case 38400 : {_uCam_Serial->write(BAUD_38400[i]);break;}
		case 57600 : {_uCam_Serial->write(BAUD_57600[i]);break;}
		case 115200 :{ _uCam_Serial->write(BAUD_115200[i]);break;}
			/* unsued speed on arduino 
			case 153600 : _uCam_Serial->write(BAUD_153600[i]);break;
			case 230400 : _uCam_Serial->write(BAUD_230400[i]);break;
			case 460800 : _uCam_Serial->write(BAUD_460800[i]);break;
			case 921600 : _uCam_Serial->write(BAUD_921600[i]);break;
			case 1228800 : _uCam_Serial->write(BAUD_1228800[i]);break;
			case 1843200 : _uCam_Serial->write(BAUD_1843200[i]);break;
			case 3686400 : _uCam_Serial->write(BAUD_3686400[i]);break;
			*/
			default :	{return(false); break;}// if we compute this line there is an error
		}
		
	}

	// Check if the response was an ACK
	byte code_commande;
	switch (baud)
		{
		case 2400 :{ code_commande=BAUD_2400[1];break;}
		case 4800 :{ code_commande=BAUD_4800[1];break;}
		case 9600 :{ code_commande=BAUD_9600[1];break;}
		case 19200 :{ code_commande=BAUD_19200[1];break;}
		case 38400 : { code_commande=BAUD_38400[1];break;}
		case 57600 : { code_commande=BAUD_57600[1];break;}
		case 115200 :{ code_commande=BAUD_115200[1];break;}
			/* unsued speed on arduino 
			case 153600 : { code_commande=BAUD_153600[1];break;}
			case 230400 : { code_commande=BAUD_230400[1];break;}
			case 460800 : { code_commande=BAUD_460800[1];break;}
			case 921600 : { code_commande=BAUD_921600[1];break;}
			case 1228800 : { code_commande=BAUD_1228800[1];break;}
			case 1843200 : { code_commande=BAUD_1843200[1];break;}
			case 3686400 : { code_commande=BAUD_3686400[1];break;}
			*/
		
		}
	if (Get_Response(_ACK,code_commande)) 
	{
		// An ACk was received so we can now set the mbed to the new baud rate
		_uCam_Serial->begin(baud); // switch output to correct speed;
		_baud=baud; // update propriety of uCam object
		// New baud rate confirmed - return true
		return(true);
	} else
	// Something went wrong - return false
	return(false);
}

boolean uCam::Initial(byte COLOUR, byte RES) {
	// Set up a buffer to hold the uCam response
	unsigned char buf[6] = {0xAA, 0x01, 0x00, COLOUR, 0x00, 0x00};

	//Amend the initial command for the type of image
	if (buf[3]==0x07) {
		buf[5] = RES;           // The image is a JPEG so set byte 5
	} else {
		buf[4] = RES;           // The image is RAW so set byte 4
	}

	// Get the number of bytes to be transferred
	// Start by finding out how many bytes per pixel
	if (COLOUR == GREY_8BIT || COLOUR == COLOUR_8BIT) {
		picture_size = 1;
	} else if (COLOUR == COLOUR_16BIT) {
		picture_size = 2;
	}
	// Then multiply this by the number of pixels
	if (RES == RAW_80x60)
	picture_size = picture_size*80*60;
	else if (RES == RAW_160x120)
	picture_size = picture_size*160*120;
	else if (RES == RAW_320x240)
	picture_size = picture_size*320*240;
	else if (RES == RAW_640x480)
	picture_size = picture_size*640*480;
	else if (RES == RAW_128x128)
	picture_size = picture_size*128*128;
	else if (RES == RAW_128x96)
	picture_size = picture_size*128*96;

	// Send out the initial command
	for (int i=0; i<6; i++) {
		_uCam_Serial->write(buf[i]);
	}

	// Check for ACK
	if (Get_Response(_ACK,0x01)) {
		// An ACk was received - return true
		return(true);
	} else {
		// Something went wrong - return false
		return(false);
	}
}

boolean uCam::Get_Picture(byte *data) 
{
	// Send out the get picture command
	for (int i=0; i<6; i++) {
		_uCam_Serial->write(GET_PICTURE[i]);
	}

	if (!Get_Response(_ACK,0x04))
	// Something went wrong
	{return(false);}
	
    // Get the reply from the uCam module (will be 6 bytes
	unsigned char reply[6] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
	unsigned long taille_image =0;
	word i=0;
	
	while (i<6) 
	{
		if (_uCam_Serial->available()>0) 
		{
			reply[i]= _uCam_Serial->read();
			if (i>2) {taille_image=(taille_image <<8)+reply[i];}
			i++;
		}
		
	}
	
	// Analyse response from uCam
	if((reply[0]!=0xAA)|(reply[1]!=0x0A)|(reply[2]!=0x02)) {return false;} // en cas d'erreur de retour
	// */
	// Get data from uCam
	i=0;
	// while (i<picture_size) 
	// while ((i<picture_size) & (i<taille_image)) 
	while (i<taille_image) // 
	{
		if (_uCam_Serial->available()>0)
		{
			data[i] = _uCam_Serial->read();
			i++;
		}
	}

	// We need a small delay (1ms) from receiving the SYNC response and sending an ACK in return
	delay(1);
	for (int i=0; i<6; i++) 
	{
		_uCam_Serial->write(ACK[i]);
	}
	// Everything is now complete so return true
	return(true);
}



